@@ -5,31 +5,17 @@ import io
 import pickle
 import re
 import tkinter as tk
-from bisect import (
-    bisect_left,
-)
+from bisect import bisect_left
 from collections import deque
-from collections.abc import (
-    Callable,
-    Generator,
-    Iterator,
-    Sequence,
-)
+from collections.abc import Callable, Generator, Hashable, Iterable, Iterator, Sequence
 from itertools import islice, repeat
 from typing import Literal
 
 from .colors import color_map
-from .formatters import (
-    to_bool,
-)
-from .other_classes import (
-    Box_nt,
-    DotDict,
-    EventDataDict,
-    Highlight,
-    Loc,
-    Span,
-)
+from .constants import symbols_set
+from .formatters import to_bool
+from .other_classes import Box_nt, DotDict, EventDataDict, Highlight, Loc, Span
+from .types import AnyIter
 
 unpickle_obj = pickle.loads
 
@@ -62,9 +48,15 @@ def get_data_from_clipboard(
     return [[data]]
 
 
+def recursive_bind(widget: tk.Misc, event: str, callback: Callable) -> None:
+    widget.bind(event, callback)
+    for child in widget.winfo_children():
+        recursive_bind(child, event, callback)
+
+
 def tksheet_type_error(kwarg: str, valid_types: list[str], not_type: object) -> str:
     valid_types = ", ".join(f"{type_}" for type_ in valid_types)
-    return f"Argument '{kwarg}' must be one of the following types: {valid_types}, " f"not {type(not_type)}."
+    return f"Argument '{kwarg}' must be one of the following types: {valid_types}, not {type(not_type)}."
 
 
 def new_tk_event(keysym: str) -> tk.Event:
@@ -73,6 +65,25 @@ def new_tk_event(keysym: str) -> tk.Event:
     return event
 
 
+def event_has_char_key(event: object) -> bool:
+    return (
+        event and hasattr(event, "char") and (event.char.isalpha() or event.char.isdigit() or event.char in symbols_set)
+    )
+
+
+def event_opens_dropdown_or_checkbox(event=None) -> bool:
+    if event is None:
+        return False
+    elif event == "rc":
+        return True
+    return (
+        (hasattr(event, "keysym") and event.keysym in {"Return", "F2", "BackSpace"})
+        or (
+            hasattr(event, "keycode") and event.keycode == "??" and hasattr(event, "num") and event.num == 1
+        )  # mouseclick
+    )
+
+
 def dropdown_search_function(
     search_for: object,
     data: Sequence[object],
@@ -225,6 +236,16 @@ def b_index(sorted_seq: Sequence[int], num_to_index: int) -> int:
     return idx
 
 
+def bisect_in(sorted_seq: Sequence[int], num: int) -> bool:
+    """
+    Faster than 'num in sorted_seq'
+    """
+    try:
+        return sorted_seq[bisect_left(sorted_seq, num)] == num
+    except Exception:
+        return False
+
+
 def get_dropdown_kwargs(
     values: list = [],
     set_value: object = None,
@@ -295,13 +316,19 @@ def is_iterable(o: object) -> bool:
         return False
 
 
-def int_x_iter(i: Iterator[int] | int) -> Iterator[int]:
+def int_x_iter(i: AnyIter[int] | int) -> AnyIter[int]:
     if isinstance(i, int):
         return (i,)
     return i
 
 
-def unpack(t: tuple[object] | tuple[Iterator[object]]) -> tuple[object]:
+def int_x_tuple(i: AnyIter[int] | int) -> tuple[int]:
+    if isinstance(i, int):
+        return (i,)
+    return tuple(i)
+
+
+def unpack(t: tuple[object] | tuple[AnyIter[object]]) -> tuple[object]:
     if not len(t):
         return t
     if is_iterable(t[0]) and len(t) == 1:
@@ -427,7 +454,7 @@ def consecutive_ranges(seq: Sequence[int]) -> Generator[tuple[int, int]]:
             yield seq[start], seq[-1] + 1
 
 
-def is_contiguous(iterable: Iterator[int]) -> bool:
+def is_contiguous(iterable: Iterable[int]) -> bool:
     itr = iter(iterable)
     prev = next(itr)
     return all(i == (prev := prev + 1) for i in itr)
@@ -500,7 +527,7 @@ def cell_right_within_box(
 
 
 def get_last(
-    it: Iterator,
+    it: AnyIter[object],
 ) -> object:
     if hasattr(it, "__reversed__"):
         try:
@@ -522,7 +549,7 @@ def index_exists(seq: Sequence[object], index: int) -> bool:
         return False
 
 
-def add_to_displayed(displayed: list[int], to_add: Iterator[int]) -> list[int]:
+def add_to_displayed(displayed: list[int], to_add: Iterable[int]) -> list[int]:
     # assumes to_add is sorted in reverse
     for i in reversed(to_add):
         ins = bisect_left(displayed, i)
@@ -535,35 +562,14 @@ def move_elements_by_mapping(
     new_idxs: dict[int, int],
     old_idxs: dict[int, int] | None = None,
 ) -> list[object]:
-    # move elements of a list around, displacing
-    # other elements based on mapping
-    # of {old index: new index, ...}
+    # move elements of a list around
+    # displacing other elements based on mapping
+    # new_idxs = {old index: new index, ...}
+    # old_idxs = {new index: old index, ...}
     if old_idxs is None:
         old_idxs = dict(zip(new_idxs.values(), new_idxs))
-
-    # create dummy list
-    res = [0] * len(seq)
-
-    # create generator of values yet to be put into res
-    remaining = (e for i, e in enumerate(seq) if i not in new_idxs)
-
-    # goes over res twice:
-    # once to put elements being moved in new spots
-    # then to fill remaining spots with remaining elements
-
-    # fill new indexes in res
-    if len(new_idxs) > int(len(seq) / 2) - 1:
-        # if moving a lot of items better to do comprehension
-        return [
-            next(remaining) if i_ not in old_idxs else e_
-            for i_, e_ in enumerate(seq[old_idxs[i]] if i in old_idxs else e for i, e in enumerate(res))
-        ]
-    else:
-        # if just moving a few items assignments are fine
-        for old, new in new_idxs.items():
-            res[new] = seq[old]
-        # fill remaining indexes
-        return [next(remaining) if i not in old_idxs else e for i, e in enumerate(res)]
+    remaining_values = (e for i, e in enumerate(seq) if i not in new_idxs)
+    return [seq[old_idxs[i]] if i in old_idxs else next(remaining_values) for i in range(len(seq))]
 
 
 def move_elements_to(
@@ -598,23 +604,15 @@ def get_new_indexes(
 
 
 def insert_items(
-    seq: list[object] | tuple[object],
+    seq: list[object],
     to_insert: dict[int, object],
     seq_len_func: Callable | None = None,
-) -> list:
-    # inserts many items into a list using a dict of reverse sorted order of
-    # {index: value, index: value, ...}
-    res = []
-    extended = 0
-    for i, (idx, v) in enumerate(reversed(to_insert.items())):
-        # need to extend seq if it's not long enough
-        if seq_len_func and idx - i > len(seq):
-            seq_len_func(idx - i - 1)
-        res.extend(seq[extended : idx - i])
-        extended += idx - i - extended
-        res.append(v)
-    res.extend(seq[extended:])
-    seq = res
+) -> list[object]:
+    if to_insert:
+        if seq_len_func and next(iter(to_insert)) >= len(seq) + len(to_insert):
+            seq_len_func(next(iter(to_insert)) - len(to_insert))
+        for idx, v in reversed(to_insert.items()):
+            seq[idx:idx] = [v]
     return seq
 
 
@@ -622,8 +620,7 @@ def data_to_displayed_idxs(
     to_convert: list[int],
     displayed: list[int],
 ) -> list[int]:
-    data_indexes = set(to_convert)
-    return [i for i, e in enumerate(displayed) if e in data_indexes]
+    return [i for i, e in enumerate(displayed) if bisect_in(to_convert, e)]
 
 
 def displayed_to_data_idxs(
@@ -640,6 +637,8 @@ def rounded_box_coords(
     y2: float,
     radius: int = 5,
 ) -> tuple[float]:
+    if y2 - y1 < 2 or x2 - x1 < 2:
+        return x1, y1, x2, y1, x2, y2, x1, y2
     return (
         x1 + radius,
         y1,
@@ -704,7 +703,106 @@ def diff_gen(seq: list[float]) -> Generator[int]:
     )
 
 
-def zip_fill_2nd_value(x: Iterator, o: object) -> Generator[object, object]:
+def gen_coords(
+    start_row: int,
+    start_col: int,
+    end_row: int,
+    end_col: int,
+    reverse: bool = False,
+) -> Generator[tuple[int, int]]:
+    if reverse:
+        for r in reversed(range(start_row, end_row)):
+            for c in reversed(range(start_col, end_col)):
+                yield (r, c)
+    else:
+        for r in range(start_row, end_row):
+            for c in range(start_col, end_col):
+                yield (r, c)
+
+
+def box_gen_coords(
+    start_row: int,
+    start_col: int,
+    total_cols: int,
+    total_rows: int,
+    reverse: bool = False,
+) -> Generator[tuple[int, int]]:
+    if reverse:
+        # yield start cell
+        yield (start_row, start_col)
+        # yield any remaining cells in the starting row before the start column
+        if start_col:
+            for col in reversed(range(start_col)):
+                yield (start_row, col)
+        # yield any cells above start row
+        for row in reversed(range(start_row)):
+            for col in reversed(range(total_cols)):
+                yield (row, col)
+        # yield cells from bottom of table upward
+        for row in range(total_rows - 1, start_row, -1):
+            for col in reversed(range(total_cols)):
+                yield (row, col)
+        # yield any remaining cells in start row
+        for col in range(total_cols - 1, start_col, -1):
+            yield (start_row, col)
+    else:
+        # Yield cells from the start position to the end of the current row
+        for col in range(start_col, total_cols):
+            yield (start_row, col)
+        # yield from the next row to the last row
+        for row in range(start_row + 1, total_rows):
+            for col in range(total_cols):
+                yield (row, col)
+        # yield from the beginning up to the start
+        for row in range(start_row):
+            for col in range(total_cols):
+                yield (row, col)
+        # yield any remaining cells in the starting row before the start column
+        for col in range(start_col):
+            yield (start_row, col)
+
+
+def next_cell(
+    start_row: int,
+    start_col: int,
+    end_row: int,
+    end_col: int,
+    row: int,
+    col: int,
+    reverse: bool = False,
+) -> tuple[int, int]:
+    if reverse:
+        col -= 1
+        if col < start_col:
+            col = end_col - 1
+            row -= 1
+            if row < start_row:
+                row = end_row - 1
+    else:
+        col += 1
+        if col == end_col:
+            col = start_col
+            row += 1
+            if row == end_row:
+                row = start_row
+    return row, col
+
+
+def is_last_cell(
+    start_row: int,
+    start_col: int,
+    end_row: int,
+    end_col: int,
+    row: int,
+    col: int,
+    reverse: bool = False,
+) -> bool:
+    if reverse:
+        return row == start_row and col == start_col
+    return row == end_row - 1 and col == end_col - 1
+
+
+def zip_fill_2nd_value(x: AnyIter[object], o: object) -> Generator[object, object]:
     return zip(x, repeat(o))
 
 
@@ -1233,13 +1331,13 @@ def span_froms(
     return from_r, from_c
 
 
-def del_named_span_options(options: dict, itr: Iterator, type_: str) -> None:
+def del_named_span_options(options: dict, itr: AnyIter[Hashable], type_: str) -> None:
     for k in itr:
         if k in options and type_ in options[k]:
             del options[k][type_]
 
 
-def del_named_span_options_nested(options: dict, itr1: Iterator, itr2: Iterator, type_: str) -> None:
+def del_named_span_options_nested(options: dict, itr1: AnyIter[Hashable], itr2: AnyIter[Hashable], type_: str) -> None:
     for k1 in itr1:
         for k2 in itr2:
             k = (k1, k2)
@@ -1320,7 +1418,7 @@ def set_align(
 def del_from_options(
     options: dict,
     key: str,
-    coords: int | Iterator | None = None,
+    coords: int | AnyIter[int | tuple[int, int]] | None = None,
 ) -> dict:
     if isinstance(coords, int):
         if coords in options and key in options[coords]:
