@@ -1,8 +1,9 @@
+from __future__ import annotations
+
+from collections.abc import Iterable, Iterator
 from typing import Tuple, Union
 
-from .other_classes import (
-    Span,
-)
+from .other_classes import Span
 
 CreateSpanTypes = Union[
     Tuple[()],
@@ -15,3 +16,5 @@ CreateSpanTypes = Union[
     Tuple[int, None, int, None, int, None, int, None],
     Span,
 ]
+
+AnyIter = Iterable | Iterator
